% computeEdgesDictionary Create dictionary for fast edge lookup
%
%   edgesDictionary = computeEdgesDictionary(edges) that contains edge indices
%       for each pair of nodes forming an edge.
%
%
%   See also: Mesh


function edgesDictionary = computeEdgesDictionary(edges)

arguments
    edges (2,:) double
end

nNodes = max(edges(:));
nEdges = size(edges, 2);

% NB: this implementation uses double amount of memory, 
% alternatively sort node indices edge-wise
edgeKeys = sub2ind([nNodes, nNodes], [edges(1,:), edges(2,:)], ...
                   [edges(2,:), edges(1,:)]);

edgesDictionary = dictionary(edgeKeys, repmat(1:nEdges, 1, 2));

end